@@ -16,6 +16,11 @@
 
 #include "config.h"
 
+/*
+ * Typedefs
+ */
+typedef unsigned short int	us_int;
+
 /*
  * Memory mapped mimes types
  */
@@ -61,6 +66,7 @@ extern const char *http_fv_srv;
 extern const char *http_fv_pro;
 extern const char *http_fv_cgi;
 extern const char *http_fv_teg;
+extern const char *http_fv_lch;
 extern const char *http_fv_con_alv;
 extern const char *http_fv_con_cls;
 
@@ -99,6 +105,9 @@ struct cfg {
 	char	sslportc[8];		/* SSL Listener Port in char */
 	char	sslcert[1024];		/* SSL Full Filename for server cert */
 	char	sslcertkey[1024];	/* SSL Full Filename for server key */
+	char	c401[1024];		/* Filename for 401 custom response */
+	char	c403[1024];		/* Filename for 403 custom response */
+	char	c404[1024];		/* Filename for 404 custom response */
 };
 extern struct cfg config;
 
@@ -106,70 +115,73 @@ extern struct cfg config;
  * Information structure for header
  */
 struct header {
-	char			rq_method[1024];
-	char			rq_uri[1024];
-	char			rq_protocol[1024];
-	char			rq_files[1024];
-	char			rq_filep[1024];
-	char			rq_filef[1024];
-	char			rq_query[1024];
-	char			rq_option[1024];
-	char			rq_script[1024];
-	char			rq_index[1024];
-	char			rq_fv_cte[1024];
-	char			rq_fv_ims[1024];
-	char			rq_fv_ref[1024];
-	char			rq_fv_agt[1024];
-	char			rq_fv_con[1024];
-	char			rq_fv_cok[1024];
-	char			rq_fv_clt[1024];
-	char			rq_fv_hos[1024];
-	char			rq_fv_aut[1024];
-	char			rq_fv_ran[1024];
-	char			rp_fv_dat[1024];
-	char			rp_fv_dam[1024];
-	char			rp_fv_cte[1024];
-	char			rp_fv_loc[1024];
-	char			rp_fv_auw[1024];
-	char			rp_header[8192];
-	int			rp_fsize;
-	int			rp_foffs;
-	int			rp_hsize;
-	unsigned short int	rp_status;
-	unsigned short int	x_opt;
-	unsigned short int	x_qry;
-	unsigned short int	x_sct;
-	unsigned short int	x_ims;
-	unsigned short int	x_ref;
-	unsigned short int	x_agt;
-	unsigned short int	x_con;
-	unsigned short int	x_cok;
-	unsigned short int	x_clt;
-	unsigned short int	x_hos;
-	unsigned short int	x_aut;
-	unsigned short int	x_ran;
-	unsigned short int	x_cgi;
-	unsigned short int	x_idx;
+	char	rq_method[1024];	/* Request Method */
+	char	rq_uri[1024];		/* Request URI */
+	char	rq_protocol[1024];	/* Request HTTP Protocol */
+	char	rq_files[1024];		/* Request Filename */
+	char	rq_filep[1024];		/* Request Path */
+	char	rq_filef[1024];		/* Request Filename with Path */
+	char	rq_query[1024];		/* Request Query String */
+	char	rq_option[1024];	/* Request Option String */
+	char	rq_script[1024];	/* Request Script String */
+	char	rq_index[1024];		/* Request Index Filename */
+	char	rq_docrootv[1024];	/* Request Virtual Docroot */
+	char	rq_fv_cte[1024];	/* Request Value Content-Type: */
+	char	rq_fv_ims[1024];	/* Request Value If-Modified-Since: */
+	char	rq_fv_ref[1024];	/* Request Value Referer: */
+	char	rq_fv_agt[1024];	/* Request Value User-Agent: */
+	char	rq_fv_con[1024];	/* Request Value Connection: */
+	char	rq_fv_cok[1024];	/* Request Value Cookie: */
+	char	rq_fv_clt[1024];	/* Request Value Content-Length: */
+	char	rq_fv_hos[1024];	/* Request Value Host: */
+	char	rq_fv_aut[1024];	/* Request Value Authorization: */
+	char	rq_fv_ran[1024];	/* Request Value Range: */
+	char	rp_fv_dat[1024];	/* Response Value Date: */
+	char	rp_fv_dam[1024];	/* Response Value ? */
+	char	rp_fv_cte[1024];	/* Response Value Content-Type: */
+	char	rp_fv_loc[1024];	/* Response Value Location: */
+	char	rp_fv_auw[1024];	/* Response Value WWW-Authenticate: */
+	char	rp_header[8192];	/* Response Header */
+	int	rp_fsize;		/* Repsonse File Size */
+	int	rp_foffs;		/* Response File Offset */
+	int	rp_hsize;		/* Response Header Size */
+	us_int	rp_status;		/* Response Status */
+	us_int	x_opt;			/* Flag for Option */
+	us_int	x_qry;			/* Flag for Query */
+	us_int	x_sct;			/* Flag for Script */
+	us_int	x_ims;			/* Flag for If-Modified-Since: */
+	us_int	x_ref;			/* Flag for Referer: */
+	us_int	x_agt;			/* Flag for User-Agent: */
+	us_int	x_con;			/* Flag for Connection: */
+	us_int	x_cok;			/* Flag for Cookie: */
+	us_int	x_clt;			/* Flag for Content-Lenght: */
+	us_int	x_hos;			/* Flag for Host: */
+	us_int	x_aut;			/* Flag for Authorization: */
+	us_int	x_ran;			/* Flag for Range: */
+	us_int	x_cgi;			/* Flag for CGI */
+	us_int	x_idx;			/* Flag for Index */
+	us_int	x_dov;			/* Flag for Virutal Host */
 };
 
 /*
- * Information structure per connection
+ * Information structure for each connection
  */
 struct connection {
-	int			to;		/* Timeout */
-	int			state;		/* 0 = close, 1 = keep alive */
-	char			ip[24];		/* Client IP */
-	char			pt[8];		/* Client Port */
-	int			pfdo;		/* Partitial, Open File Desc */
-	int			pfdn[HDN];	/* Partitial, File Desc */
-	int			pfds[HDN];	/* Partitial, Bytes sent */
-	char			*plreq[HDN];	/* Partitial, Log Request */
-	char			*plref[HDN];	/* Partitial, Log Referer */
-	char			*plagt[HDN];	/* Partitial, Log Client */
-	char			*pllog[HDN];	/* Partitial, Log File */
-	SSL			*ssl;		/* SSL Object */
-	unsigned short int	x_ssl;		/* SSL Connection Flag */
-	unsigned short int	x_ip6;		/* IPv6 Connection Flag */
-	unsigned short int	status;		/* Response status */
+	int	to;			/* Timeout */
+	us_int	state;			/* 0 = close, 1 = keep alive */
+	char	ip[24];			/* Client IP */
+	char	pt[8];			/* Client Port */
+	int	pfdo;			/* Partitial, Open File Desc */
+	int	pfdn[HDN];		/* Partitial, File Desc */
+	int	pfds[HDN];		/* Partitial, Bytes sent */
+	us_int	status[HDN];		/* Response status */
+	char	*plreq[HDN];		/* Partitial, Log Request */
+	char	*plref[HDN];		/* Partitial, Log Referer */
+	char	*plagt[HDN];		/* Partitial, Log Client */
+	char	*pllog[HDN];		/* Partitial, Log File */
+	SSL	*ssl;			/* SSL Object */
+	us_int	x_ssl;			/* Flag for SSL Connection */
+	us_int	x_ip6;			/* Flag for IPv6 Connection */
+	us_int	x_chk[HDN];		/* Flag for Chunked Data */
 };
 extern struct connection c[];
