@@ -140,7 +140,8 @@ http_verify(const char *header, const char *cip, const int sfd)
  *	0 = close connection, 1 = keep alive connection, -1 = error
  */
 int
-http_proc(const char *header, char *body, const int hr, const int sfd)
+http_proc(const char *header, char *body, const int hr, const int blen,
+    const int sfd)
 {
 	DIR		*odir;
 	int		cpid, file, fds1[2], fds2[2];
@@ -160,7 +161,7 @@ http_proc(const char *header, char *body, const int hr, const int sfd)
 	bs = BS;
 
 	/* parse request header */
-	if ((rh = http_header(header, NULL, 0, sfd)) == NULL)
+	if ((rh = http_header(header, NULL, 0, blen, sfd)) == NULL)
 		return -1;
 
 	/* check parse response */
@@ -432,7 +433,7 @@ http_proc(const char *header, char *body, const int hr, const int sfd)
 
 		/* if post send data to cgis stdin */
 		if (strcmp(rh->rq_method, "POST") == 0)
-			write(fds1[1], body, strlen(body));
+			sys_write(fds1[1], body, blen);
 		/* close fd to cgi stdin */
 		close(fds1[1]);
 
@@ -454,7 +455,7 @@ http_proc(const char *header, char *body, const int hr, const int sfd)
 				    sizeof(status))) > 0) {
 					free(rh);
 					rh = http_header(header, status, s,
-					    sfd);
+					    blen, sfd);
 				}
 				len = rh->rp_hsize + strlen(b) + 1;
 				if ((x = malloc(len)) == NULL) {
@@ -805,49 +806,33 @@ http_cgi_header(char *header_cgi, char *status, const int dsize1,
  * 	0 = headers not complete, <nr of headers> = headers complete
  */
 int
-http_header_comp(const char *header)
+http_header_comp(char *header)
 {
-	int	i, l, r, foot, head;
-	char	line[1024], status[1024], *footer = "\r\n\r\n";
-	char	*p, *p_copy;
+	int	r, foot, head;
+	char	*p;
 
 	r = foot = head = 0;
 
-	if (strlen(header) == 0)
-		return r;
-
-	if ((p = strdup(header)) == NULL)
-		return -1;
-	p_copy = p;
-
-	l = strcutl(line, p, 1, sizeof(line));
-
 	/* count all headers */
-	for (i = 1; i <= l; i++) {
-		strcutl(line, p, i, sizeof(line));
-		status[0] = '\0';
-		if (strcutw(status, line, 3, sizeof(status)) != -1) {
-			if (strncmp("HTTP/", status, 5) == 0)
-				head++;
-		}
+	p = header;
+	while ((p = strstr(p, "HTTP/")) != NULL) {
+		head++;
+		p = p + 5;
 	}
 
 	/* count all footers */
-	while ((p = strstr(p, footer)) != NULL) {
+	p = header;
+	while ((p = strstr(p, "\r\n\r\n")) != NULL) {
 		foot++;
-		p = p + strlen(footer);
-		if (strncmp("POST ", header, 5) == 0) {
-			p = NULL;
+		p = p + 4;
+		if (strncmp("POST ", header, 5) == 0)
 			break;
-		}
 	}
 
 	/* headers match footers */
 	if (head == foot)
 		r = head;
 
-	free(p_copy);
-
 	return r;
 }
 
@@ -855,38 +840,35 @@ http_header_comp(const char *header)
  * http_body_comp()
  *	check if received body arrived complete
  * Return:
- *	0 = body not complete, 1 = body complete
+ *	0 = body not complete, <bytes of body> = body complete
  */
 int
-http_body_comp(const char *header, char *body)
+http_body_comp(const char *header, char *body, const int blen)
 {
-	int	i, r, len;
+	int	i, r, hlen;
 	char	line[1024], option[1024], clength[16];
 	
 	r = 0;
 
-	/* get body size */
-	len = strlen(body);
-
-	/* cut out body termination crap */
-	if (body[len - 2] == '\r') {
-		body[len - 2] = '\0';
-		len -= 2;
-	} else if (body[len - 1] == '\n') {
-		body[len - 1] = '\0';
-		len -= 1;
-	}
-
-	/* get Content-Length: from header */
+	/* get content length from header */
 	for (i = 2; strcutl(line, header, i, sizeof(line)) != -1; i++) {
 		strcutw(option, line, 1, sizeof(option));
-		if (strcasecmp(option, http_fn_clt) == 0)
+		if (strcasecmp(option, http_fn_clt) == 0) {
 			strcuts(clength, line, ' ', '\0', sizeof(clength));
+			break;
+		}
 	}
+	hlen = atoi(clength);
 
-	/* some browsers count final '\n' to Content-Length, some not, sigh */
-	if (len == atoi(clength) || len + 1 == atoi(clength))
-		r = 1;
+	/* handle the post body termination mess */
+	if (blen >= hlen) {
+		if (body[blen - 2] == '\r')
+			r = blen - 2;
+		else if (body[blen - 1] == '\n')
+			r = blen - 1;
+		else
+			r = blen;
+	}
 
 	return r;
 }
@@ -1177,7 +1159,7 @@ http_body(const char *title, const char *msg, const char *header)
  */
 struct header *
 http_header(const char *header_data, const char *force_status,
-    const int force_status_nr, const int sfd)
+    const int force_status_nr, const int blen, const int sfd)
 {
 	int		fsize, fd, i, r;
 	char		line[1024], option[1024];
@@ -1622,8 +1604,10 @@ http_header(const char *header_data, const char *force_status,
 			setenv("SERVER_PORT", config.serverportc, 1);
 
 		/* optional */
-		if (h->x_clt)
-			setenv("CONTENT_LENGTH", h->rq_fv_clt, 1);
+		if (h->x_clt) {
+			snprintf(tmp, sizeof(tmp), "%d", blen);
+			setenv("CONTENT_LENGTH", tmp, 1);
+		}
 		if (h->x_cok)
 			setenv("HTTP_COOKIE", h->rq_fv_cok, 1);
 		if (h->x_con)
