@@ -98,7 +98,7 @@ const char *http_fn_aut		= "Authorization:";
 const char *http_fn_auw		= "WWW-Authenticate:";
 const char *http_fn_ran		= "Range:";
 const char *http_fn_rac		= "Content-Range:";
-const char *http_fv_srv		= "Nostromo/1.7.2 (Unix)";
+const char *http_fv_srv		= "Nostromo/1.7.3 (Unix)";
 const char *http_fv_pro		= "HTTP/1.1";
 const char *http_fv_cgi		= "CGI/1.1";
 const char *http_fv_teg		= "chunked";
@@ -280,7 +280,7 @@ send_file(const int sfd, const int hr)
  * main()
  *	Nostromo webserver
  * Return:
- *	0
+ *	0 = terminated successfull, 1 = terminated with error
  */
 int
 main(int argc, char *argv[])
@@ -300,6 +300,7 @@ main(int argc, char *argv[])
 	struct sockaddr_in6	sa6, ca6;
 	struct in6_addr		in6addr_any = IN6ADDR_ANY_INIT;
 	struct timeval		tv;
+	socklen_t		ssize;
 	fd_set			master_r, master_w, set_r, set_w, seto_r;
 	time_t			tnow;
 	SSL_CTX			*ssl_ctx = NULL;
@@ -475,7 +476,7 @@ main(int argc, char *argv[])
 	 */
 	if (config.user[0] != '0') {
 		if ((pwd = getpwnam(config.user)) == NULL)
-			err(1, "getpwnam");
+			errx(1, "%s: no such user.", config.user);
 	}
 
 	/*
@@ -887,15 +888,15 @@ main(int argc, char *argv[])
 			if (rd > 0 && (sdnow == sdlis1 || sdnow == sdlis2 ||
 			    sdnow == sdlis3 || sdnow == sdlis4)) {
 				if (sdnow == sdlis3 || sdnow == sdlis4) {
-					size = sizeof(ca6);
+					ssize = sizeof(ca6);
 					if ((sdnew = accept(sdnow,
-					    (struct sockaddr *)&ca6, &size)) ==
+					    (struct sockaddr *)&ca6, &ssize)) ==
 					    -1)
 						continue;
 				} else {
-					size = sizeof(ca);
+					ssize = sizeof(ca);
 					if ((sdnew = accept(sdnow,
-					    (struct sockaddr *)&ca, &size)) ==
+					    (struct sockaddr *)&ca, &ssize)) ==
 					    -1)
 						continue;
 				}
@@ -1007,15 +1008,12 @@ main(int argc, char *argv[])
 			memset(body, 0, sizeof(body));
 
 			/* receive data */
-			if (c[sdnow].x_ssl) {
-				if ((rt = SSL_read(c[sdnow].ssl, in,
-				    sizeof(in) - 1)) < 1)
-					goto quit;
-			} else {
-				if ((rt = sys_read(sdnow, in, sizeof(in) - 1))
-				    < 1)
-					goto quit;
-			}
+			if (c[sdnow].x_ssl)
+				rt = SSL_read(c[sdnow].ssl, in, sizeof(in) - 1);
+			else
+				rt = sys_read(sdnow, in, sizeof(in) - 1);
+			if (rt < 1)
+				goto quit;
 			in[rt] = '\0';
 
 			/* set timestamp for connection timeout */
@@ -1024,7 +1022,7 @@ main(int argc, char *argv[])
 			/* receive everything */
 			FD_ZERO(&seto_r);
 			FD_SET(sdnow, &seto_r);
-			while (http_header_comp(in) == 0) {
+			while ((size = http_header_comp(in)) == 0) {
 				tv.tv_sec = 1;
 				tv.tv_usec = 0;
 				r = select(sdnow + 1, &seto_r, NULL, NULL, &tv);
@@ -1035,18 +1033,8 @@ main(int argc, char *argv[])
 				}
 				if (r == 0)
 					goto quit;
-				if (c[sdnow].x_ssl) {
-					if ((rp = SSL_read(c[sdnow].ssl, tmp,
-					    sizeof(tmp) - 1)) < 1)
-						goto quit;
-				} else {
-					if ((rp = sys_read(sdnow, tmp,
-					    sizeof(tmp) - 1)) < 1)
-						goto quit;
-				}
-				rt += rp;
 				/* size check for data */
-				if (rt > sizeof(in) - 1) {
+				if (rt == sizeof(in) - 1) {
 					h = http_head(http_s_413, "-",
 					    c[sdnow].ip);
 					b = http_body(http_s_413, "", h);
@@ -1060,12 +1048,21 @@ main(int argc, char *argv[])
 					free(b);
 					goto quit;
 				}
-				strlcat(in, tmp, sizeof(in));
-				memset(tmp, 0, sizeof(tmp));
+				if (c[sdnow].x_ssl) {
+					rp = SSL_read(c[sdnow].ssl, in + rt,
+					    (sizeof(in) - rt) - 1);
+				} else {
+					rp = sys_read(sdnow, in + rt,
+					    (sizeof(in) - rt) - 1);
+				}
+				if (rp < 1)
+					goto quit;
+				rt += rp;
 			}
+			in[rt] = '\0';
 
 			/* size check for header count */
-			if ((r = http_header_comp(in)) > HDN) {
+			if (size > HDN) {
 				h = http_head(http_s_413, "-", c[sdnow].ip);
 				b = http_body(http_s_413, "", h);
 				if (c[sdnow].x_ssl) {
@@ -1085,7 +1082,7 @@ main(int argc, char *argv[])
 				post = 0;
 
 			/* separate every header */
-			for (i = 0, j = 0, k = 0; in[i] != '\0'; i++) {
+			for (i = 0, j = 0, k = 0; i < rt; i++) {
 				/* size check for header */
 				if (j == sizeof(header[k]) - 1) {
 					h = http_head(http_s_413, "-",
@@ -1118,15 +1115,15 @@ main(int argc, char *argv[])
 				}
 			}
 
-			/* if post, receive body entity */
+			/* for post receive body entity */
 			if (post) {
-				if (tmp[0] != '\0') {
-					strlcpy(body, tmp, sizeof(body));
-					memset(tmp, 0, sizeof(tmp));
-				}
+				rt = j;
+				memcpy(body, tmp, rt);
+				memset(tmp, 0, sizeof(tmp));
 				FD_ZERO(&seto_r);
 				FD_SET(sdnow, &seto_r);
-				while(http_body_comp(header[0], body) != 1) {
+				while((size = http_body_comp(header[0], body,
+				    rt)) == 0) {
 					tv.tv_sec = 1;
 					tv.tv_usec = 0;
 					r = select(sdnow + 1, &seto_r, NULL,
@@ -1138,18 +1135,8 @@ main(int argc, char *argv[])
 					}
 					if (r == 0)
 						goto quit;
-					if (c[sdnow].x_ssl) {
-						if ((rp = SSL_read(c[sdnow].ssl,
-						    tmp, sizeof(tmp) - 1)) < 1)
-							goto quit;
-					} else {
-						if ((rp = sys_read(sdnow, tmp,
-						    sizeof(tmp) - 1)) < 1)
-							goto quit;
-					}
-					rt += rp;
 					/* size check for body */
-					if (rt > sizeof(body) - 1) {
+					if (rt == sizeof(body) - 1) {
 						h = http_head(http_s_413, "-",
 						    c[sdnow].ip);
 						b = http_body(http_s_413,"", h);
@@ -1164,8 +1151,18 @@ main(int argc, char *argv[])
 						free(b);
 						goto quit;
 					}
-					strlcat(body, tmp, sizeof(body));
-					memset(tmp, 0, sizeof(tmp));
+					if (c[sdnow].x_ssl) {
+						rp = SSL_read(c[sdnow].ssl,
+						    body + rt,
+						    (sizeof(body) - rt) - 1);
+					} else {
+						rp = sys_read(sdnow,
+						    body + rt,
+						    (sizeof(body) - rt) - 1);
+					}
+					if (rp < 1)
+						goto quit;
+					rt += rp;
 				}
 			}
 
@@ -1173,7 +1170,7 @@ main(int argc, char *argv[])
 			for (i = 0; i < k; i++) {
 				r = http_verify(header[i], c[sdnow].ip, sdnow);
 				if (r == 1)
-					r = http_proc(header[i], body, i,
+					r = http_proc(header[i], body, i, size,
 					    sdnow);
 				/* on error abort processing */
 				if (r == -1) {
