@@ -26,12 +26,13 @@ void		send_block(const int, const int);
  * http.c
  */
 int		http_verify(const char *, const char *, const int);
-int		http_proc(const char *, char *, const int, const int);
+int		http_proc(const char *, char *, const int, const int,
+		    const int);
 int		http_cgi_getexec(char *, char *, const char *, const int,
 		    const int);
 int		http_cgi_header(char *, char *, const int, const int);
-int		http_header_comp(const char *);
-int		http_body_comp(const char *, char *);
+int		http_header_comp(char *);
+int		http_body_comp(const char *, char *, const int);
 int		http_access_htaccess(char *, const char *, const int);
 int		http_alog(const int, const int);
 char *		http_chunk(const char *);
@@ -39,7 +40,8 @@ char *		http_date(struct tm *);
 char *		http_uridecode(const char *);
 char *		http_head(const char *, const char *, const char *);
 char *		http_body(const char *, const char *, const char *);
-struct header *	http_header(const char *, const char *, const int, const int);
+struct header *	http_header(const char *, const char *, const int, const int,
+		    const int);
 
 /*
  * sys.c
