@@ -93,7 +93,7 @@ sys_access_auth(const char *clientuser, const char *clientpw)
 	int		i, r, fd, found;
 	char		ch[1], *password;
 	char		serveruserpw[1024], serveruser[1024], serverpw[1024];
-	unsigned char	salt[2];
+	char		salt[2];
 
 	i = r = found = 0;
 
